@@ -545,7 +545,7 @@ class CA( BiologicProgram ):
 
             params[ ch ] = {
                 'Voltage_step':      ch_params[ 'voltages' ],
-                'vs_inital':         [ ch_params[ 'vs_initial' ] ]* steps,
+                'vs_initial':        [ ch_params[ 'vs_initial' ] ]* steps,
                 'Duration_step':     ch_params[ 'durations' ],
                 'Step_number':       steps - 1,
                 'Record_every_dT':   ch_params[ 'time_interval' ],
@@ -678,7 +678,7 @@ class CP( BiologicProgram ):
 
             params[ ch ] = {
                 'Current_step':      ch_params[ 'currents' ],
-                'vs_inital':         [ ch_params[ 'vs_initial' ] ]* steps,
+                'vs_initial':        [ ch_params[ 'vs_initial' ] ]* steps,
                 'Duration_step':     ch_params[ 'durations' ],
                 'Step_number':       steps - 1,
                 'Record_every_dT':   ch_params[ 'time_interval' ],
@@ -817,6 +817,7 @@ class CALimit( BiologicProgram ):
 
         channels = kwargs[ 'channels' ] if ( 'channels' in kwargs ) else None
         params = set_defaults( params, defaults, channels )
+
         super().__init__(
             device,
             params,
@@ -853,7 +854,7 @@ class CALimit( BiologicProgram ):
 
             datum.voltage,
             datum.current,
-            datum.voltage* datum.current, # power
+            datum.voltage* datum.current,  # power
             datum.cycle
         )
 
@@ -869,7 +870,7 @@ class CALimit( BiologicProgram ):
 
             params[ ch ] = {
                 'Voltage_step':      ch_params[ 'voltages' ],
-                'vs_inital':         [ ch_params[ 'vs_initial' ] ]* steps,
+                'vs_initial':        [ ch_params[ 'vs_initial' ] ]* steps,
                 'Duration_step':     ch_params[ 'durations' ],
                 'Step_number':       steps - 1,
                 'Record_every_dT':   ch_params[ 'time_interval' ],
@@ -1200,7 +1201,7 @@ class JV_Scan( BiologicProgram ):
             params[ ch ] = {
                 'vs_initial':   [ False ]* 5,
                 'Voltage_step': voltage_profile,
-                'Scan_Rate':    [ ch_params[ 'rate' ]* 10e-3 ]* 5,
+                'Scan_Rate':    [ ch_params[ 'rate' ] ]* 5,
                 'Scan_number':  2,
                 'Record_every_dE':   ch_params[ 'step' ],
                 'Average_over_dE':   ch_params[ 'average' ],
@@ -1269,8 +1270,9 @@ class MPP_Tracking( CALimit ):
             self.v_mpp = { ch: init_vmpp for ch in channels }
             self.probe_steps = { ch: params[ 'probe_step' ] for ch in channels }
 
-            params[ 'durations' ] = params[ 'run_time' ]
             params[ 'voltages' ]  = [ init_vmpp ]
+            params[ 'durations' ] = params[ 'run_time' ]
+            params[ 'time_interval' ] = params[ 'record_interval' ]
 
 
         super().__init__(
@@ -1389,7 +1391,7 @@ class MPP_Tracking( CALimit ):
         while True:
             # loop until measurement ends
 
-            if ( # stop signal received
+            if (  # stop signal received
                 self._stop_event is not None
                 and self._stop_event.is_set()
             ):
@@ -1408,8 +1410,6 @@ class MPP_Tracking( CALimit ):
                 self._hold_and_retrieve( hold_time )
             )
 
-            # self._append_data( hold_segments )  # add data
-
             if len( self.active_channels ) is 0:
                 # program end
                 break
@@ -1425,8 +1425,6 @@ class MPP_Tracking( CALimit ):
                 self._hold_and_retrieve( probe_time )
             )
 
-            self._append_data( probe_segments ) # add data
-
             if len( self.active_channels ) is 0:
                 # program end
                 break
@@ -1470,30 +1468,6 @@ class MPP_Tracking( CALimit ):
         return ( active, segments )
 
 
-    def _append_data( self, segments ):
-        fields = lambda datum, segment: (
-            dp.calculate_time(
-                datum.t_high,
-                datum.t_low,
-                segment.info,
-                segment.values
-            ),
-
-            datum.voltage,
-            datum.current,
-            datum.voltage* datum.current,  # power
-            datum.cycle
-        )
-
-        for ch, segment in segments.items():
-            data = [
-                self._fields( *fields( datum, segment ) )
-                for datum in segment.data
-            ]
-
-            self._data[ ch ] += data
-
-
     def _calculate_powers( self, hold_segments, probe_segments ):
         powers = {
             ch: self._calculate_power(
